<?php

namespace Drupal\standwithukraine\Plugin\Block;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\standwithukraine\StandWithUkraineTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block to display a message on the background of the flag.
 *
 * @Block(
 *   id = "standwithukraine_block",
 *   admin_label = @Translation("StandWithUkraine"),
 * )
 */
class StandWithUkraineBlock extends BlockBase implements StandWithUkraineBlockInterface {

  use StandWithUkraineTrait;

  /**
   * The default values.
   */
  const CONFIGURATION = [
    'single' => FALSE,
    'url' => NULL,
  ];

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return self::CONFIGURATION + [
      'inherit' => array_fill_keys(
        array_keys(self::CONFIGURATION),
        self::INHERIT
      ),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $this->alter($form, $this->configuration);

    $link = Link::createFromRoute(
      $this->t('the settings page'),
      'standwithukraine.settings',
      [],
      [
        'attributes' => [
          'target' => '_blank',
        ],
      ]
    );

    $url = $this->configFactory->get('standwithukraine.settings')->get('url');
    $weight = 0;

    foreach (array_keys(self::CONFIGURATION) as $name) {
      $form[$name]['#weight'] = ++$weight;

      $form[$sub_name = 'inherit_' . $name] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Inherit @field', [
          '@field' => strtolower($form[$name]['#title']),
        ]),
        '#description' => $this->t(
          'Re-use value from the %field field on @url.',
          [
            '%field' => $form[$name]['#title'],
            '@url' => $link->toString(),
          ]
        ),
        '#default_value' => $this->configuration['inherit'][$name],
        '#disabled' => empty($url),
        '#weight' => ++$weight,
      ];

      if ($form[$sub_name]['#disabled']) {
        if ($form[$sub_name]['#default_value']) {
          $form[$sub_name]['#default_value'] = FALSE;
        }
      }
      else {
        $form[$name]['#states'] = [
          'disabled' => [
            ':input[name="settings[' . $sub_name . ']"]' => [
              'checked' => TRUE,
            ],
          ],
        ];
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    foreach (array_keys(self::CONFIGURATION) as $key) {
      $parents = [];

      foreach ([$key, 'inherit'] as $parent) {
        NestedArray::setValue(
          $this->configuration,
          $parents = array_merge([$parent], $parents),
          $form_state->getValue(implode('_', $parents))
        );
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->configFactory->get('standwithukraine.settings');
    $options = [];

    foreach (self::CONFIGURATION as $key => $default_value) {
      $global_value = $config->get($key);

      $options['#' . $key] = $this->configuration['inherit'][$key] &&
        ($default_value !== NULL || !empty($global_value))
        ? $global_value : $this->configuration[$key];
    }

    return [
      '#theme' => 'standwithukraine',
      '#attached' => [
        'library' => ['standwithukraine/common'],
      ],
    ] + $options;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    return Cache::mergeTags(parent::getCacheTags(), ['standwithukraine']);
  }

}
